# -*-makefile-*-
#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2009 NXP B.V.,
# All Rights Reserved.
#
# Filename: sde/linux/Kbuild.mk
#
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#   1 20070319   batelaan    Initial, from STB225 project.
#   2 20070713   batelaan    Move included components info into SDE2 makefiles.
#   3 20070720   batelaan    Deprecate KMODULE and KERNEL_MODE vars
#   4 20070917   batelaan    Detect incorrect modules names
#   5 20071022   batelaan    Stop on first
#   6 20080513   batelaan    Add more debugging
#   7 20080528   batelaan    Change KCONFIG_VALUE definition.
#   8 20080707   batelaan    Build kernel modules as external modules.
#   9 20080718   batelaan    Cope with no modules being built at all.
#  10 20090209   batelaan    Drop KMODULE_KBUILD_GENERATED_FILE var passing to sub-build.
#  11 20090316   batelaan    Update name/location of generated Kbuild file.

# Reset:
obj-y=
obj-m=
obj-=

# Are we building or cleaning?
ifeq "$(filter scripts/Makefile.build,$(MAKEFILE_LIST))" ""
  # Cleaning. This hack will ensure that in the definition of KMODULE_COMPS below
  # every component is included, even if CONFIG_... variables are undefined.
  OR_CLEANING:=true
endif

# Variable: <KMODULE_NAMES>
# List of kernel modules/drivers. These are the names of the
# .ko files produced, if they are not built-ins.
#
# Order determines module init function invocation
# order for those modules that are built-ins!

# Macro: KCONFIG_VALUE
# Return config value for building component (argument 1).
# Should return y, m, or an empty string.
KCONFIG_VALUE ?= $(strip $(_$(strip $1)_KCONFIG))

# Macro: KCONFIG_VALUE_EX
# Debug version of <KCONFIG_VALUE>
KCONFIG_VALUE_EX = $(strip \
  $(call KCONFIG_VALUE,$1) \
  $(if $(KMODULE_DEBUG_BUILD),$(warning KCONFIG_VALUE $1 = $(call KCONFIG_VALUE,$1))) \
)

# Variable: KMODULE_COMP_NAME_xxx
# Translates from a kernel driver/module name to a SDE2 component name.
# Optional variable; if not defined then they are the same.
# For example:
# (start code)
# KMODULE_COMP_NAME_phStbOsal = tmosal
# (end code)
#
# See: KMODULE_COMP_NAME

KMODULE_COMPS = $(foreach s,$(KMODULE_NAMES),$(KMODULE_COMPS_$s))

# Function: KMODULE_ENVVARS
# Returns a string defining environment variables for building a component.
# This includes:-
# _TMDIVERSITY. _kmodule_ is added if the argument 1 is m, otherwise _kmode_.
#
# Usage:-
# - $(call KMODULE_ENVVARS,m)
# - $(call KMODULE_ENVVARS,y)
KMODULE_ENVVARS=_TMDIVERSITY=$(if $(filter m,$1),_kmodule,_kmode)$(if $(_TMDIVERSITY),$(_TMDIVERSITY),_)

# Are we building or cleaning?
ifneq "$(filter scripts/Makefile.build,$(MAKEFILE_LIST))" ""
  # Building
  # Workout if we need to rebuild the generated KBuild files,
  # which should only be done on the first pass. We always want to do
  # this (on the first pass) so that we invoke the SDE2 system to check
  # that the SDE2 component builds are up to date.
  ifneq "$(shell ls 2>/dev/null $(THIS_DIR)/.secondPass)" ""
    firstPass=true
  endif
  ifeq "$(shell ls 2>/dev/null $(THIS_DIR)/.firstPass)" ""
    firstPass=true
  endif

  ifeq "$(firstPass)" "true"
    $(if $(KMODULE_DEBUG_BUILD),$(warning First Pass))
    $(shell rm -f $(THIS_DIR)/.*Pass)
    $(shell touch $(THIS_DIR)/.firstPass)
  else
    $(if $(KMODULE_DEBUG_BUILD),$(warning Second Pass))
    $(shell touch $(THIS_DIR)/.secondPass)
  endif

  $(if $(KMODULE_DEBUG_BUILD),$(warning KBUILD_BUILTIN=$(KBUILD_BUILTIN) KBUILD_MODULES=$(KBUILD_MODULES)))

# Variable: KMODULE_SELECT
# Selects what should be built.
# - if equal to y, then built-in modules should be built.
# - if equal to m, then loadable modules should be built.
# - otherwise nothing should be built.
KMODULE_SELECT?=y

# Function: KMODULE_SELECTED
# Tests if a kernel module should be built during this phase of the kernel build process.
# If true, then return a non-empty string, otherwise an empty string.
#
# Usage: $(call KMODULE_SELECTED,myKernelModuleName)
KMODULE_SELECTED=$(filter $(KMODULE_SELECT),$(call KCONFIG_VALUE_EX,$1))

# Function: KMODULE_COMP_NAME
# Returns the main SDE2 component name for a kernel module.
# By default this is the same as the kernel module name,
# unless <KMODULE_COMP_NAME_xxx> is defined, in which case it is the value of that variable.
#
# Usage: $(call KMODULE_COMP_NAME,myKernelModuleName)
KMODULE_COMP_NAME=$(firstword $(KMODULE_COMP_NAME_$1) $1)

# Function: KMODULE_COMP_DIR
# Returns the directory path of the main SDE2 component for a kernel module.
# Uses <KMODULE_COMP_NAME> to obtain the component name first.
#
# Usage: $(call KMODULE_COMP_DIR,myKernelModuleName)
KMODULE_COMP_DIR=$(_$(call KMODULE_COMP_NAME,$1)_DIR)

  ifeq "$(firstPass)" "true"
    # Need to know directories where SDE2 components are located:
    include $(_TMTGTBUILDROOT)/project/loc_list.mk

    # Function to be called with name of module (phStbOsal, phStbCore, uda3000 etc)
    define DRIVER_SET_ACTIONS
      # Only build if module's CONFIG param value matches Linux kernel build phase (builtins or modules):
      $(if $(KMODULE_DEBUG_BUILD),$(warning $1 KMODULE_SELECTED returns $(call KMODULE_SELECTED,$1)))
      $(if $(KMODULE_DEBUG_BUILD),$$(warning $1 - CONFIG value is $(call KCONFIG_VALUE,$1), $(if $(call KMODULE_SELECTED,$1),,NOT )built in this phase))
      ifneq "$(call KMODULE_SELECTED,$1)" ""
        KMODULE_HAS_BEEN_SELECTED=true
	$(shell $(if $(KMODULE_DEBUG_BUILD)$(_TMECHO)$V,set -x;) rm -f $(THIS_DIR)/$1/Kbuild 1>&2; \
		printf '\n#### Building $1 module ($(if $(filter m,$(call KCONFIG_VALUE,$1)),as loadable module,built-in)) ####\n\n' 1>&2; \
		$(call KMODULE_ENVVARS,$(call KCONFIG_VALUE,$1)) $(MAKE) -C $(call KMODULE_COMP_DIR,$1) \
			kmodule_generate_Kbuild_file \
			KMODULE_NAME=$1 \
			KMODULE_BUILD_DIR=$(THIS_DIR) 1>&2 || \
		echo '$$(error Failed to build $1 module)' >$(THIS_DIR)/$1/Kbuild)
        include $(THIS_DIR)/$1/Kbuild
      endif
    endef
  else
    define DRIVER_SET_ACTIONS
      # Only build if module's CONFIG param value matches Linux kernel build phase (builtins or modules):
      ifneq "$(call KMODULE_SELECTED,$1)" ""
        include $(THIS_DIR)/$1/Kbuild
      endif
    endef
  endif

else

  # Cleaning
  # Function to be called with name of module (phStbOsal, phStbCore, uda3000 etc)
  define DRIVER_SET_ACTIONS
    .DEFAULT clean __clean: clean_sde2_comps_$1
    clean_sde2_comps_$1:
	@printf '\n\n#### Cleaning $1 drivers ####\n\n';
	$(Q)-$(call KMODULE_ENVVARS,y) $(MAKE) -C $(call KMODULE_COMP_DIR,$1) clean
	$(Q)-$(call KMODULE_ENVVARS,m) $(MAKE) -C $(call KMODULE_COMP_DIR,$1) clean
  endef
endif

__ignore:=$(foreach m,$(KMODULE_NAMES),\
  $(if $(call KMODULE_SELECTED,$m)$(OR_CLEANING),\
    $(eval $(call DRIVER_SET_ACTIONS,$m))\
  )\
)

# if no inbuilt drivers have been built, we need to have at least something to build,
# otherwise we get:
# mipsel-linux-ld: drivers/nxp/built-in.o: No such file: No such file or directory
ifeq "$(firstPass)" "true"
  ifndef KMODULE_HAS_BEEN_SELECTED
    $(warning obj-y = $(obj-y))
    obj-y += drivers_nxp_dummy.o
    $(THIS_DIR)/drivers_nxp_dummy.c drivers/nxp/drivers_nxp_dummy.c:
	@test -f $@ || echo 'int drivers_nxp_dummy;' >$@
  endif
endif
